@@ -292,10 +292,14 @@ IdentificationTextGenerator::generateVolumeIdentificationText(IdentificationStri
                          */
                         bool limitMapIndicesFlag = false;
                         switch (ciftiFile->getDataFileType()) {
+                            case DataFileTypeEnum::ANNOTATION:
+                                break;
                             case DataFileTypeEnum::BORDER:
                                 break;
                             case DataFileTypeEnum::CONNECTIVITY_DENSE:
                                 break;
+                            case DataFileTypeEnum::CONNECTIVITY_DENSE_DYNAMIC:
+                                break;
                             case DataFileTypeEnum::CONNECTIVITY_DENSE_LABEL:
                                 break;
                             case DataFileTypeEnum::CONNECTIVITY_DENSE_PARCEL:
@@ -466,10 +470,14 @@ IdentificationTextGenerator::generateSurfaceIdentificationText(IdentificationStr
              */
             bool limitMapIndicesFlag = false;
             switch (cmdf->getDataFileType()) {
+                case DataFileTypeEnum::ANNOTATION:
+                    break;
                 case DataFileTypeEnum::BORDER:
                     break;
                 case DataFileTypeEnum::CONNECTIVITY_DENSE:
                     break;
+                case DataFileTypeEnum::CONNECTIVITY_DENSE_DYNAMIC:
+                    break;
                 case DataFileTypeEnum::CONNECTIVITY_DENSE_LABEL:
                     break;
                 case DataFileTypeEnum::CONNECTIVITY_DENSE_PARCEL:
