@@ -32,6 +32,8 @@
 #include <QFile>
 #include "zlib.h"
 
+#include <algorithm>
+
 using namespace caret;
 using namespace std;
 
@@ -42,6 +44,7 @@ namespace caret
     class ZFileImpl : public CaretBinaryFile::ImplInterface
     {
         gzFile m_zfile;
+        const static int64_t CHUNK_SIZE;
     public:
         ZFileImpl() { m_zfile = NULL; }
         void open(const QString& filename, const CaretBinaryFile::OpenMode& opmode);
@@ -52,11 +55,14 @@ namespace caret
         void write(const void* dataIn, const int64_t& count);
         ~ZFileImpl();
     };
+    
+    const int64_t ZFileImpl::CHUNK_SIZE = 1<<26;//64MiB, large enough for good performance, small enough for zlib, must convert to uint32
 #endif //ZLIB_VERSION
 
     class QFileImpl : public CaretBinaryFile::ImplInterface
     {
         QFile m_file;
+        const static int64_t CHUNK_SIZE;
     public:
         void open(const QString& filename, const CaretBinaryFile::OpenMode& opmode);
         void close();
@@ -65,6 +71,8 @@ namespace caret
         void read(void* dataOut, const int64_t& count, int64_t* numRead);
         void write(const void* dataIn, const int64_t& count);
     };
+    
+    const int64_t QFileImpl::CHUNK_SIZE = 1<<30;//1GiB, QT4 apparently chokes at more than 2GiB via buffer.read using int32
 }
 
 CaretBinaryFile::ImplInterface::~ImplInterface()
@@ -166,14 +174,22 @@ void ZFileImpl::open(const QString& filename, const CaretBinaryFile::OpenMode& o
 #endif
     if (m_zfile == NULL)
     {
-        throw DataFileException("error opening compressed file '" + filename + "'");
+        if (!QFile::exists(filename))
+        {
+            if (!(opmode & CaretBinaryFile::TRUNCATE))
+            {
+                throw DataFileException("failed to open compressed file '" + filename + "', file does not exist, or folder permissions prevent seeing it");
+            } else {//use same logic as QFile impl for now
+                throw DataFileException("failed to open compressed file '" + filename + "', unable to create file");
+            }
+        }//TODO: check gzerror and errno for more informative error messages
+        throw DataFileException("failed to open compressed file '" + filename + "'");
     }
 }
 
 void ZFileImpl::close()
 {
     if (m_zfile == NULL) return;//happens when closed and then destroyed, error opening
-    gzflush(m_zfile, Z_FULL_FLUSH);
     if (gzclose(m_zfile) != 0) throw DataFileException("error closing compressed file '" + m_fileName + "'");
     m_zfile = NULL;
 }
@@ -181,17 +197,12 @@ void ZFileImpl::close()
 void ZFileImpl::read(void* dataOut, const int64_t& count, int64_t* numRead)
 {
     if (m_zfile == NULL) throw DataFileException("read called on unopened ZFileImpl");//shouldn't happen
-    const int64_t CHUNK_SIZE = (1<<26);//64MB, should be large enough for good performance, and small enough not to give zlib trouble - needs to convert to unsigned int
     int64_t totalRead = 0;
     int readret = 0;//to preserve the info of the read that broke early
     while (totalRead < count)
     {
-        int64_t iterSize = count - totalRead;
-        if (iterSize > CHUNK_SIZE)
-        {
-            iterSize = CHUNK_SIZE;
-        }
-        readret = gzread(m_zfile, (uint8_t*)dataOut + totalRead, iterSize);
+        int64_t iterSize = min(count - totalRead, CHUNK_SIZE);
+        readret = gzread(m_zfile, ((uint8_t*)dataOut) + totalRead, iterSize);
         if (readret < 1) break;//0 or -1 indicate eof or error
         totalRead += readret;
     }
@@ -232,16 +243,11 @@ int64_t ZFileImpl::pos()
 void ZFileImpl::write(const void* dataIn, const int64_t& count)
 {
     if (m_zfile == NULL) throw DataFileException("read called on unopened ZFileImpl");//shouldn't happen
-    const int64_t CHUNK_SIZE = (1<<26);//64MB, should be large enough for good performance, and small enough not to give zlib trouble - needs to convert to unsigned int
     int64_t totalWritten = 0;
     while (totalWritten < count)
     {
-        int64_t iterSize = count - totalWritten;
-        if (iterSize > CHUNK_SIZE)
-        {
-            iterSize = CHUNK_SIZE;
-        }
-        int writeret = gzwrite(m_zfile, (uint8_t*)dataIn + totalWritten, iterSize);
+        int64_t iterSize = min(count - totalWritten, CHUNK_SIZE);
+        int writeret = gzwrite(m_zfile, ((uint8_t*)dataIn) + totalWritten, iterSize);
         if (writeret < 1) break;//0 or -1 indicate eof or error
         totalWritten += writeret;
     }
@@ -254,11 +260,11 @@ ZFileImpl::~ZFileImpl()
     {
         close();
     } catch (CaretException& e) {//handles DataFileException, should be the only culprit
-        CaretLogWarning(e.whatString());
+        CaretLogSevere(e.whatString());
     } catch (exception& e) {
-        CaretLogWarning(e.what());
+        CaretLogSevere(e.what());
     } catch (...) {
-        CaretLogWarning("caught unknown exception type while closing a compressed file");
+        CaretLogSevere("caught unknown exception type while closing a compressed file");
     }
 }
 #endif //ZLIB_VERSION
@@ -274,7 +280,22 @@ void QFileImpl::open(const QString& filename, const CaretBinaryFile::OpenMode& o
     m_file.setFileName(filename);
     if (!m_file.open(mode))
     {
-        throw DataFileException("failed to open file '" + m_fileName + "'");
+        if (!m_file.exists())
+        {
+            if (!(opmode & CaretBinaryFile::TRUNCATE))
+            {
+                throw DataFileException("failed to open file '" + filename + "', file does not exist, or folder permissions prevent seeing it");
+            } else {//m_file.error() doesn't help identify this case, see below
+                throw DataFileException("failed to open file '" + filename + "', unable to create file");
+            }
+        }
+        switch (m_file.error())
+        {
+            case QFile::ResourceError://on linux at least, it never gives another code besides the unhelpful OpenError
+                throw DataFileException("failed to open file '" + filename + "', too many open files");
+            default:
+                throw DataFileException("failed to open file '" + filename + "'");
+        }
     }
 }
 
@@ -285,16 +306,24 @@ void QFileImpl::close()
 
 void QFileImpl::read(void* dataOut, const int64_t& count, int64_t* numRead)
 {
-    int64_t readret = m_file.read((char*)dataOut, count);//expect QFile to handle it without manual chunking
+    int64_t total = 0;
+    int64_t readret = -1;
+    while (total < count)
+    {
+        int64_t maxToRead = min(count - total, CHUNK_SIZE);
+        readret = m_file.read(((char*)dataOut) + total, maxToRead);//QFile chokes on large reads also
+        if (readret < 1) break;//0 or -1 means error or eof
+        total += readret;
+    }
     if (numRead == NULL)
     {
-        if (readret != count)
+        if (total != count)
         {
             if (readret < 0) throw DataFileException("error while reading file '" + m_fileName + "'");
             throw DataFileException("premature end of file in '" + m_fileName + "'");
         }
     } else {
-        *numRead = readret;
+        *numRead = total;
     }
 }
 
@@ -310,14 +339,21 @@ int64_t QFileImpl::pos()
 
 void QFileImpl::write(const void* dataIn, const int64_t& count)
 {
-    int64_t writeret = m_file.write((const char*)dataIn, count);//again, expect QFile to handle it in one shot
+    int64_t total = 0;
+    int64_t writeret = -1;
+    while (total < count)
+    {
+        int64_t maxToWrite = min(count - total, CHUNK_SIZE);
+        writeret = m_file.write((const char*)dataIn, maxToWrite);//QFile probably also chokes on large writes
+        if (writeret < 1) break;//0 or -1 means error or eof
+        total += writeret;
+    }
     const AString msg = ("failed to write file '"
                          + m_fileName
                          + "'.  Tried to write "
                          + AString::number(count)
                          + " bytes but actually wrote "
-                         + AString::number(writeret)
+                         + AString::number(total)
                          + " bytes.");
-    if (writeret != count) throw DataFileException(msg);
-    //if (writeret != count) throw DataFileException("failed to write to file '" + m_fileName + "'");
+    if (total != count) throw DataFileException(msg);
 }
