@@ -35,6 +35,7 @@
 
 
 #include <cmath>
+#include <limits>
 
 #include "MathFunctions.h"
 
@@ -571,6 +572,23 @@ MathFunctions::subtractVectors(
     resultOut[2] = v1[2] - v2[2];
 }
 
+/**
+ * subtract vectors (3d)  result = v1 - v2.
+ * @param v1  1st vector input
+ * @param v2  2nd vector input
+ * @return  3D vector containing result of subtraction.
+ *
+ */
+void
+MathFunctions::subtractVectors(const double v1[3],
+                               const double v2[3],
+                               double resultOut[3])
+{
+    resultOut[0] = v1[0] - v2[0];
+    resultOut[1] = v1[1] - v2[1];
+    resultOut[2] = v1[2] - v2[2];
+}
+
 void MathFunctions::addVectors(const float v1[3], const float v2[3], float resultOut[3])
 {
     resultOut[0] = v1[0] + v2[0];
@@ -645,13 +663,12 @@ MathFunctions::dotProduct(
  * @return     Dot product of the two vectors.
  *
  */
-float
+double
 MathFunctions::dotProduct(
                           const double p1[3],
                           const double p2[3])
 {
-    float dot = p1[0]*p2[0] + p1[1]*p2[1] + p1[2]*p2[2];
-    return dot;
+    return p1[0]*p2[0] + p1[1]*p2[1] + p1[2]*p2[2];
 }
 
 /**
@@ -1259,6 +1276,26 @@ MathFunctions::arraysEqual(
     return true;
 }
 
+/**
+ * Get the average of two coordinates.
+ * @param c1 - coordinate 1
+ * @param c2 - coordinate 2
+ * @param outputAverage A three-dimensional array into
+ * which the average of the two coordinates is
+ * placed.
+ *
+ */
+void
+MathFunctions::averageOfTwoCoordinates(const float c1[3],
+                                       const float c2[3],
+                                       float outputAverage[3])
+{
+    outputAverage[0] = (c1[0] + c2[0]) / 2.0f;
+    outputAverage[1] = (c1[1] + c2[1]) / 2.0f;
+    outputAverage[2] = (c1[2] + c2[2]) / 2.0f;
+}
+
+
 /**
  * Get the average of three coordinates.
  * @param c1 - coordinate 1
@@ -1309,6 +1346,29 @@ MathFunctions::averageOfThreeCoordinates(
     outputAverage[outputOffset+2] = (xyzAll[offsetCoord1+2] + xyzAll[offsetCoord2+2] + xyzAll[offsetCoord3+2]) / 3.0f;
 }
 
+/**
+ * Get the average of four coordinates.
+ * @param c1 - coordinate 1
+ * @param c2 - coordinate 2
+ * @param c3 - coordinate 1
+ * @param c4 - coordinate 2
+ * @param outputAverage A three-dimensional array into
+ * which the average of the four coordinates is
+ * placed.
+ *
+ */
+void
+MathFunctions::averageOfFourCoordinates(const float c1[3],
+                                        const float c2[3],
+                                        const float c3[3],
+                                        const float c4[3],
+                                        float outputAverage[3])
+{
+    outputAverage[0] = (c1[0] + c2[0] + c3[0] + c4[0]) / 4.0f;
+    outputAverage[1] = (c1[1] + c2[1] + c3[1] + c4[1]) / 4.0f;
+    outputAverage[2] = (c1[2] + c2[2] + c3[2] + c4[2]) / 4.0f;
+}
+
 /**
  * Angle formed by p1, p2, p3 (angle at p2).  Returned angle is in radians.
  * This method uses Java Math.acos() and produces highly accurate results.
@@ -1553,7 +1613,7 @@ uint32_t MathFunctions::gcd(uint32_t num1, uint32_t num2)
 
 bool MathFunctions::isInf(const float number)
 {
-    return (abs(number) > 1.0f && number * 2.0f == number);
+    return (abs(number) == numeric_limits<float>::infinity());
 }
 
 bool MathFunctions::isNaN(const float number)
@@ -1563,7 +1623,7 @@ bool MathFunctions::isNaN(const float number)
 
 bool MathFunctions::isNegInf(const float number)
 {
-    return (number < -1.0f && number * 2.0f == number);
+    return (number == -numeric_limits<float>::infinity());
 }
 
 bool MathFunctions::isNumeric(const float number)
@@ -1713,16 +1773,16 @@ bool MathFunctions::matrixToQuatern(const float matrix[3][3], float cijk[4])
 bool MathFunctions::matrixToQuatern(const double matrix[3][3], double cijk[4])
 {//formulas from http://en.wikipedia.org/wiki/Rotation_matrix#Quaternion
     const float toler = 0.0001f;
-    float ivec[3] = { matrix[0][0], matrix[1][0], matrix[2][0] };
-    float jvec[3] = { matrix[0][1], matrix[1][1], matrix[2][1] };
-    float kvec[3] = { matrix[0][2], matrix[1][2], matrix[2][2] };
+    double ivec[3] = { matrix[0][0], matrix[1][0], matrix[2][0] };
+    double jvec[3] = { matrix[0][1], matrix[1][1], matrix[2][1] };
+    double kvec[3] = { matrix[0][2], matrix[1][2], matrix[2][2] };
     if (!(std::abs(1.0f - normalizeVector(ivec)) <= toler)) return false;//use the "not less than or equal to" trick to catch NaNs
     if (!(std::abs(1.0f - normalizeVector(jvec)) <= toler)) return false;
     if (!(std::abs(1.0f - normalizeVector(kvec)) <= toler)) return false;
     if (!(dotProduct(ivec, jvec) <= toler)) return false;
     if (!(dotProduct(ivec, kvec) <= toler)) return false;
     if (!(dotProduct(jvec, kvec) <= toler)) return false;
-    float tempvec[3];
+    double tempvec[3];
     crossProduct(ivec, jvec, tempvec);
     if (!(dotProduct(tempvec, kvec) >= 0.9f)) return false;//i cross j must be k, otherwise it contains a flip
     int method = 0;
