@@ -354,6 +354,8 @@ namespace caret {
          */         
         virtual const GiftiLabelTable* getMapLabelTable(const int32_t mapIndex) const = 0;
         
+        bool isMedialWallLabelInMapLabelTable(const int32_t mapIndex) const;
+        
         /**
          * Get the palette normalization modes that are supported by the file.
          *
@@ -426,7 +428,7 @@ namespace caret {
         virtual bool isModifiedExcludingPaletteColorMapping() const;
         
         /* documented in cxx file. */
-        bool isModifiedPaletteColorMapping() const;
+        virtual bool isModifiedPaletteColorMapping() const;
         
         /* documented in cxx file. */
         bool isModified() const;
