@@ -31,10 +31,14 @@
 #include "DataFileException.h"
 #include "FileInformation.h"
 #include "GiftiMetaData.h"
+#include "ImageCaptureSettings.h"
 #include "ImageFile.h"
+#include "SceneClass.h"
 
 using namespace caret;
 
+const float ImageFile::s_defaultWindowDepthPercentage = 990;
+
 /**
  * Constructor.
  */
@@ -43,7 +47,8 @@ ImageFile::ImageFile()
 {
     m_fileMetaData.grabNew(new GiftiMetaData());
     
-    m_image = new QImage();
+    m_image   = new QImage();
+    m_windowZ = s_defaultWindowDepthPercentage;
 }
 
 /**
@@ -57,6 +62,7 @@ ImageFile::ImageFile(const QImage& qimage)
     m_fileMetaData.grabNew(new GiftiMetaData());
     
     m_image = new QImage(qimage);
+    m_windowZ = s_defaultWindowDepthPercentage;
 }
 
 /**
@@ -83,6 +89,7 @@ ImageFile::ImageFile(const unsigned char* imageDataRGBA,
     m_image = new QImage(imageWidth,
                              imageHeight,
                              QImage::Format_RGB32);
+    m_windowZ = s_defaultWindowDepthPercentage;
     
     bool isOriginAtTop = false;
     switch (imageOrigin) {
@@ -1010,6 +1017,21 @@ ImageFile::resizeToWidth(const int32_t width)
                                       Qt::SmoothTransformation);
 }
 
+/**
+ * Resize the image to the given height while preserving the aspect ratio
+ * of the image.
+ *
+ * @param height
+ *     Height for image.
+ */
+void
+ImageFile::resizeToHeight(const int32_t height)
+{
+    CaretAssert(m_image);
+    *m_image = m_image->scaledToHeight(height,
+                                       Qt::SmoothTransformation);
+}
+
 /**
  * Resize the image so that its width is no larger than the given value.
  * If the image's current width is less than the given value, no
@@ -1149,6 +1171,130 @@ ImageFile::getImageBytesRGBA(const IMAGE_DATA_ORIGIN_LOCATION imageOrigin,
     return false;
 }
 
+bool
+ImageFile::getImageResizedBytes(const IMAGE_DATA_ORIGIN_LOCATION imageOrigin,
+                                const int32_t resizeToWidth,
+                                const int32_t resizeToHeight,
+                                std::vector<uint8_t>& bytesRGBAOut) const
+{
+    bytesRGBAOut.clear();
+    
+    if (m_image == NULL) {
+        return false;
+    }
+    
+    const int32_t numBytes = resizeToWidth * resizeToHeight;
+    if (numBytes <= 0) {
+        return false;
+    }
+    
+    const int32_t colorComponentsPerByte = 4;
+    bytesRGBAOut.resize(numBytes * colorComponentsPerByte);
+    
+    QImage scaledImage = m_image->scaled(resizeToWidth,
+                                   resizeToHeight,
+                                   Qt::IgnoreAspectRatio,
+                                   Qt::SmoothTransformation);
+
+    /*
+     * QImage::scaled() failed.
+     */
+    if (scaledImage.isNull()) {
+        return false;
+    }
+    
+    bool isOriginAtTop = false;
+    switch (imageOrigin) {
+        case IMAGE_DATA_ORIGIN_AT_BOTTOM:
+            isOriginAtTop = false;
+            break;
+        case IMAGE_DATA_ORIGIN_AT_TOP:
+            isOriginAtTop = true;
+            break;
+    }
+    
+    /*
+     * Documentation for QImage states that setPixel may be very costly
+     * and recommends using the scanLine() method to access pixel data.
+     */
+    for (int y = 0; y < resizeToHeight; y++) {
+        const int scanLineIndex = (isOriginAtTop
+                                   ? y
+                                   : resizeToHeight -y - 1);
+        const uchar* scanLine = scaledImage.scanLine(scanLineIndex);
+        QRgb* rgbScanLine = (QRgb*)scanLine;
+        
+        for (int x = 0; x < resizeToWidth; x++) {
+            const int32_t contentOffset = (((y * resizeToWidth) * 4)
+                                           + (x * 4));
+            QRgb& rgb = rgbScanLine[x];
+            CaretAssertVectorIndex(bytesRGBAOut, contentOffset + 3);
+            bytesRGBAOut[contentOffset] = static_cast<uint8_t>(qRed(rgb));
+            bytesRGBAOut[contentOffset+1] = static_cast<uint8_t>(qGreen(rgb));
+            bytesRGBAOut[contentOffset+2] = static_cast<uint8_t>(qBlue(rgb));
+            bytesRGBAOut[contentOffset+3] = 255;
+        }
+    }
+    return true;
+}
+
+/**
+ * @return width of image (zero if image is invalid)
+ */
+int32_t
+ImageFile::getWidth() const
+{
+    int32_t w = 0;
+    
+    if (m_image != NULL) {
+        w = m_image->width();
+    }
+    
+    return w;
+}
+
+/**
+ * @return height of image (zero if image is invalid)
+ */
+int32_t
+ImageFile::getHeight() const
+{
+    int32_t h = 0;
+    
+    if (m_image != NULL) {
+        h = m_image->height();
+    }
+    
+    return h;
+}
+
+/**
+ * @return
+ *    The Window Z for the image.
+ *
+ * Note:  1000 is at far clipping plane
+ *       -1000 is at near clipping plane
+ */
+float
+ImageFile::getWindowZ() const
+{
+    return m_windowZ;
+}
+
+/**
+ * Set the Window Z for the image.
+ *
+ * Note:  1000 is at far clipping plane
+ *       -1000 is at near clipping plane
+ *
+ * @param windowZ
+ *     New window Z for drawing image.
+ */
+void
+ImageFile::setWindowZ(const float windowZ)
+{
+    m_windowZ = windowZ;
+}
 
 /**
  * Essentially writes the image file to a byte array using the given format.
@@ -1215,4 +1361,47 @@ ImageFile::setImageFromByteArray(const QByteArray& byteArray,
     }
 }
 
+/**
+ * Save file data from the scene.  For subclasses that need to
+ * save to a scene, this method should be overriden.  sceneClass
+ * will be valid and any scene data should be added to it.
+ *
+ * @param sceneAttributes
+ *    Attributes for the scene.  Scenes may be of different types
+ *    (full, generic, etc) and the attributes should be checked when
+ *    restoring the scene.
+ *
+ * @param sceneClass
+ *     sceneClass to which data members should be added.
+ */
+void
+ImageFile::saveFileDataToScene(const SceneAttributes* /*sceneAttributes*/,
+                                   SceneClass* sceneClass)
+{
+    sceneClass->addFloat("m_windowZ",
+                         m_windowZ);
+}
+
+/**
+ * Restore file data from the scene.  For subclasses that need to
+ * restore from a scene, this method should be overridden. The scene class
+ * will be valid and any scene data may be obtained from it.
+ *
+ * @param sceneAttributes
+ *    Attributes for the scene.  Scenes may be of different types
+ *    (full, generic, etc) and the attributes should be checked when
+ *    restoring the scene.
+ *
+ * @param sceneClass
+ *     sceneClass for the instance of a class that implements
+ *     this interface.  Will NEVER be NULL.
+ */
+void
+ImageFile::restoreFileDataFromScene(const SceneAttributes* /*sceneAttributes*/,
+                                        const SceneClass* sceneClass)
+{
+    m_windowZ = sceneClass->getFloatValue("m_windowZ",
+                                          s_defaultWindowDepthPercentage);
+}
+
 
