@@ -95,6 +95,19 @@ public:
                            int32_t& widthOut,
                            int32_t& heightOut) const;
     
+    bool getImageResizedBytes(const IMAGE_DATA_ORIGIN_LOCATION imageOrigin,
+                              const int32_t resizeToWidth,
+                              const int32_t resizeToHeight,
+                              std::vector<uint8_t>& bytesRGBAOut) const;
+    
+    int32_t getWidth() const;
+    
+    int32_t getHeight() const;
+    
+    float getWindowZ() const;
+    
+    void setWindowZ(const float windowZ);
+    
     virtual void readFile(const AString& filename);
     
     virtual void writeFile(const AString& filename);
@@ -121,6 +134,8 @@ public:
     
     void resizeToMaximumHeight(const int32_t maximumHeight);
     
+    void resizeToHeight(const int32_t height);
+    
     void resizeToWidth(const int32_t width);
     
     void getImageInByteArray(QByteArray& byteArrayOut,
@@ -139,6 +154,11 @@ public:
     static void getImageFileFilters(std::vector<AString>& imageFileFilters,
                                     AString& defaultFilter);
 
+    virtual void saveFileDataToScene(const SceneAttributes* sceneAttributes,
+                                     SceneClass* sceneClass);
+    
+    virtual void restoreFileDataFromScene(const SceneAttributes* sceneAttributes,
+                                          const SceneClass* sceneClass);
 private:
     ImageFile(const ImageFile&);
     
@@ -155,7 +175,11 @@ private:
     
     QImage* m_image;
     
+    float m_windowZ;
+    
     CaretPointer<GiftiMetaData> m_fileMetaData;
+    
+    static const float s_defaultWindowDepthPercentage;
 };
 
 } // namespace
