@@ -36,9 +36,14 @@
 #include "FileInformation.h"
 #include "GiftiMetaData.h"
 #include "Scene.h"
+#include "SceneAttributes.h"
+#include "SceneClass.h"
+#include "SceneClassArray.h"
 #include "SceneFileSaxReader.h"
 #include "SceneInfo.h"
+#include "SceneXmlElements.h"
 #include "SceneWriterXml.h"
+#include "SpecFile.h"
 #include "XmlSaxParser.h"
 #include "XmlWriter.h"
 
@@ -57,6 +62,8 @@ using namespace caret;
 SceneFile::SceneFile()
 : CaretDataFile(DataFileTypeEnum::SCENE)
 {
+    m_balsaStudyID = "";
+    m_baseDirectory = "";
     m_metadata = new GiftiMetaData();
 }
 
@@ -85,6 +92,9 @@ SceneFile::clear()
     
     m_metadata->clear();
     
+    m_balsaStudyID = "";
+    m_baseDirectory = "";
+    
     for (std::vector<Scene*>::iterator iter = m_scenes.begin();
          iter != m_scenes.end();
          iter++) {
@@ -154,6 +164,8 @@ SceneFile::insertScene(Scene* newScene,
     }
     
     m_scenes = tempSceneVector;
+    
+    setModified();
 }
 
 
@@ -203,12 +215,33 @@ SceneFile::getNumberOfScenes() const
  *     Scene at the given index.
  */
 Scene* 
-SceneFile::getSceneAtIndex(const int32_t indx)
+SceneFile::getSceneAtIndex(const int32_t indx) const
 {
     CaretAssertVectorIndex(m_scenes, indx);
     return m_scenes[indx];
 }
 
+/**
+ * Get the index of the given scene.
+ *
+ * @param scene
+ *     Scene for which index is requested.
+ * @return
+ *     Index of the scene or negative if scene not found.
+ */
+int32_t
+SceneFile::getIndexOfScene(const Scene* scene) const
+{
+    const int32_t numScenes = getNumberOfScenes();
+    for (int32_t i = 0; i < numScenes; i++) {
+        if (scene == getSceneAtIndex(i)) {
+            return i;
+        }
+    }
+
+    return -1;
+}
+
 /**
  * Get the scene with the given name.
  * @param sceneName
@@ -230,7 +263,6 @@ SceneFile::getSceneWithName(const AString& sceneName)
     return NULL;
 }
 
-
 /**
  * Remove the given scene.
  * @param scene
@@ -264,6 +296,75 @@ SceneFile::removeSceneAtIndex(const int32_t indx)
     removeScene(scene);
 }
 
+/**
+ * Remove the scene at the given index, but return its pointer rather than deleting it.
+ * @param index
+ *     Index of the scene.
+ */
+Scene* SceneFile::releaseScene(const int32_t& index)
+{
+    CaretAssertVectorIndex(m_scenes, index);
+    Scene* scene = m_scenes[index];
+    m_scenes.erase(m_scenes.begin() + index);
+    return scene;
+}
+
+/**
+ * Move the scene in the file by the given change in index.
+ * 
+ * @param scene
+ *     Scene that is moved.
+ * @param indexDelta
+ *     Change of index of scene in the file.
+ */
+void
+SceneFile::moveScene(Scene* scene,
+                     const int32_t indexDelta)
+{
+    if (indexDelta == 0) {
+        return;
+    }
+    
+    const int32_t numberOfScenes = getNumberOfScenes();
+    if (numberOfScenes == 1) {
+        return;
+    }
+    
+    const int32_t sceneIndex = getIndexOfScene(scene);
+    
+    const int32_t newSceneIndex = sceneIndex + indexDelta;
+    
+    if ((sceneIndex >= 0)
+        && (sceneIndex < numberOfScenes)) {
+        if (newSceneIndex < sceneIndex) {
+            if (newSceneIndex >= 0) {
+                /*
+                 * Remove scene from its index
+                 * insert scene into its new index
+                 */
+                m_scenes.erase(m_scenes.begin() + sceneIndex);
+                m_scenes.insert(m_scenes.begin() + newSceneIndex,
+                                scene);
+                setModified();
+            }
+        }
+        else {
+            CaretAssert(newSceneIndex > sceneIndex);
+            
+            if (newSceneIndex < numberOfScenes) {
+                /*
+                 * Innsert scene into its new index
+                 * Remove scene from its index
+                 */
+                m_scenes.insert(m_scenes.begin() + newSceneIndex + 1,
+                                scene);
+                m_scenes.erase(m_scenes.begin() + sceneIndex);
+                setModified();
+            }
+        }
+    }
+}
+
 /**
  * Reorder the scenes given the newly ordered scenes.
  * Any existing scenes not in the newly ordered scenes are 
@@ -303,6 +404,28 @@ SceneFile::reorderScenes(std::vector<Scene*>& orderedScenes)
     setModified();
 }
 
+int32_t SceneFile::getSceneIndexFromNumberOrName(const AString& numberOrName)
+{
+    bool ok = false;
+    int32_t ret = numberOrName.toInt(&ok) - 1;//compensate for 1-indexing that command line parsing uses
+    if (ok)
+    {
+        if (ret < 0 || ret >= getNumberOfScenes())
+        {
+            return -1;
+        }
+        return ret;
+    } else {//DO NOT search by name if the string was parsed as an integer correctly, or some idiot who names their maps as integers will get confused
+            //when getting map "12" out of a file after the file expands to more than 12 elements suddenly does something different
+        const int32_t numScenes = getNumberOfScenes();
+        for (int32_t i = 0; i < numScenes; i++) {
+            if (numberOrName == getSceneAtIndex(i)->getName()) {
+                return i;
+            }
+        }
+    }
+    return -1;
+}
 
 /**
  * @return The structure for this file.
@@ -342,6 +465,55 @@ SceneFile::getFileMetaData() const
     return m_metadata;
 }
 
+/**
+ * @return The BALSA Study ID.
+ */
+AString
+SceneFile::getBalsaStudyID() const
+{
+    return m_balsaStudyID;
+}
+
+/**
+ * Set the BALSA Study ID.
+ *
+ * @param balsaStudyID
+ *     New value for BALSA Study ID.
+ */
+void
+SceneFile::setBalsaStudyID(const AString& balsaStudyID)
+{
+    if (balsaStudyID != m_balsaStudyID) {
+        m_balsaStudyID = balsaStudyID;
+        setModified();
+    }
+}
+
+/**
+ * @return The Base Directory
+ */
+AString
+SceneFile::getBaseDirectory() const
+{
+    return m_baseDirectory;
+}
+
+/**
+ * Set the Base Directory.
+ *
+ * @param baseDirectory
+ *     New value for Base Directory.
+ */
+void
+SceneFile::setBaseDirectory(const AString& baseDirectory)
+{
+    if (baseDirectory != m_baseDirectory) {
+        m_baseDirectory = baseDirectory;
+        setModified();
+    }
+}
+
+
 /**
  * Read the scene file.
  * @param filenameIn
@@ -468,6 +640,10 @@ SceneFile::writeFile(const AString& filename)
          * Write the scene info directory
          */
         xmlWriter.writeStartElement(SceneFile::XML_TAG_SCENE_INFO_DIRECTORY_TAG);
+        xmlWriter.writeElementCData(SceneXmlElements::SCENE_INFO_BALSA_STUDY_ID_TAG,
+                                    getBalsaStudyID());
+        xmlWriter.writeElementCData(SceneXmlElements::SCENE_INFO_BASE_DIRECTORY_TAG,
+                                    getBaseDirectory());
         for (int32_t i = 0; i < numScenes; i++) {
             m_scenes[i]->getSceneInfo()->writeSceneInfo(xmlWriter,
                                                         i);
@@ -512,11 +688,53 @@ SceneFile::addToDataFileContentInformation(DataFileContentInformation& dataFileI
     
     const int32_t numScenes = getNumberOfScenes();
     if (numScenes > 0) {
-        AString sceneNamesText = "SCENE NAMES";
+        AString sceneNamesText = "Scenes:";
         for (int32_t i = 0; i < numScenes; i++) {
             const Scene* scene = getSceneAtIndex(i);
-            sceneNamesText.appendWithNewLine("    "
-                                             + scene->getName());
+            sceneNamesText.appendWithNewLine("#" + AString::number(i + 1) + "  " +
+                                             scene->getName());
+            if (dataFileInformation.isOptionFlag(DataFileContentInformation::OPTION_SHOW_MAP_INFORMATION))
+            {
+                sceneNamesText += ":";
+                const SceneAttributes* myAttrs = scene->getAttributes();
+                const SceneClass* guiMgrClass = scene->getClassWithName("guiManager");
+                if (guiMgrClass == NULL)
+                {
+                    sceneNamesText.appendWithNewLine("missing guiManager class");
+                    continue;
+                }
+                const SceneClass* sessMgrClass = guiMgrClass->getClass("m_sessionManager");
+                if (sessMgrClass == NULL)
+                {
+                    sceneNamesText.appendWithNewLine("missing m_sessionManager class");
+                    continue;
+                }
+                const SceneClassArray* brainArray = sessMgrClass->getClassArray("m_brains");
+                if (brainArray == NULL)
+                {
+                    sceneNamesText.appendWithNewLine("missing m_brains class array");
+                    continue;
+                }
+                const int numBrainClasses = brainArray->getNumberOfArrayElements();
+                for (int j = 0; j < numBrainClasses; ++j)
+                {
+                    const SceneClass* brainClass = brainArray->getClassAtIndex(j);
+                    const SceneClass* specClass = brainClass->getClass("specFile");
+                    if (specClass == NULL)
+                    {
+                        sceneNamesText.appendWithNewLine("missing specFile class in m_brains element " + AString::number(j));
+                        continue;
+                    }
+                    SpecFile tempSpec;
+                    tempSpec.restoreFromScene(myAttrs, specClass);
+                    std::vector<AString> tempNames = tempSpec.getAllDataFileNamesSelectedForLoading();
+                    int numNames = (int)tempNames.size();
+                    for (int k = 0; k < numNames; ++k)
+                    {
+                        sceneNamesText.appendWithNewLine("        " + tempNames[k]);
+                    }
+                }
+            }
             
         }
         
