@@ -158,7 +158,8 @@ SpecFile::initializeSpecFile()
     this->metadata = new GiftiMetaData();
     
     std::vector<DataFileTypeEnum::Enum> allEnums;
-    DataFileTypeEnum::getAllEnums(allEnums, false);
+    DataFileTypeEnum::getAllEnums(allEnums,
+                                  DataFileTypeEnum::OPTIONS_NONE);
     
     /*
      * Do surface files first since they need to be loaded before other files
@@ -301,6 +302,10 @@ SpecFile::addCaretDataFile(CaretDataFile* caretDataFile)
 {
     CaretAssert(caretDataFile);
     
+    if ( ! SpecFile::isDataFileTypeAllowedInSpecFile(caretDataFile->getDataFileType())) {
+        return;
+    }
+    
     /*
      * Matches to first file found that has matching name and data file type
      */
@@ -471,7 +476,7 @@ SpecFile::addDataFile(const DataFileTypeEnum::Enum dataFileType,
  *   True if the file is a member of the spec file and is written
  *   into the spec file.
  * @return
- *   SpecFileDataFile that was created or matched.  NULL if error.
+ *   SpecFileDataFile that was created or matched.  NULL if error or file type not allowed.
  *
  * @throws DataFileException
  *   If data file type is UNKNOWN.
@@ -484,37 +489,21 @@ SpecFile::addDataFilePrivate(const DataFileTypeEnum::Enum dataFileType,
                              const bool fileSavingSelectionStatus,
                              const bool specFileMemberStatus)
 {
+    if ( ! SpecFile::isDataFileTypeAllowedInSpecFile(dataFileType)) {
+        return NULL;
+    }
+    
     AString name = filename;
 
     const bool dataFileOnNetwork = DataFile::isFileOnNetwork(name);
-    const bool specFileOnNetwork = DataFile::isFileOnNetwork(getFileName());
 
-    if (dataFileOnNetwork) {
-        /* nothing */
-    }
-    else if (specFileOnNetwork) {
-        const int32_t lastSlashIndex = getFileName().lastIndexOf("/");
-        if (lastSlashIndex >= 0) {
-            const AString newName = (getFileName().left(lastSlashIndex)
-                                     + "/"
-                                     + name);
-            name = newName;
+    //NOTE: the spec file's location is completely irrelevant to this operation!
+    if (!dataFileOnNetwork) {
+        FileInformation fileInfo(name);
+        if (fileInfo.isRelative()) {
+            name = fileInfo.getAbsoluteFilePath();
         }
-        else {
-            CaretAssert(0);
-        }
-    }
-    else {
-        FileInformation specFileInfo(getFileName());
-        if (specFileInfo.isAbsolute()) {
-            FileInformation fileInfo(name);
-            if (fileInfo.isRelative()) {
-                FileInformation fileInfo(specFileInfo.getPathName(),
-                                         name);
-                name = fileInfo.getAbsoluteFilePath();
-            }
-        }
-    }
+    }//if it is on the network, don't modify it
     
 //    const AString message = ("After adding, " 
 //                             + filename
@@ -572,26 +561,26 @@ SpecFile::addDataFilePrivate(const DataFileTypeEnum::Enum dataFileType,
     return NULL; // will never get here since exception thrown
 }
 
-/**
- * @return ALL of the connectivity file types (NEVER delete contents of returned vector.
- */
-void 
-SpecFile::getAllConnectivityFileTypes(std::vector<SpecFileDataFile*>& connectivityDataFilesOut)
-{
-    connectivityDataFilesOut.clear();
-    
-    for (std::vector<SpecFileDataFileTypeGroup*>::const_iterator iter = dataFileTypeGroups.begin();
-         iter != dataFileTypeGroups.end();
-         iter++) {
-        SpecFileDataFileTypeGroup* dataFileTypeGroup = *iter;
-        if (DataFileTypeEnum::isConnectivityDataType(dataFileTypeGroup->getDataFileType())) {
-            const int32_t numFiles = dataFileTypeGroup->getNumberOfFiles();
-            for (int32_t i = 0; i < numFiles; i++) {
-                connectivityDataFilesOut.push_back(dataFileTypeGroup->getFileInformation(i));
-            }
-        }
-    }
-}
+///**
+// * @return ALL of the connectivity file types (NEVER delete contents of returned vector.
+// */
+//void 
+//SpecFile::getAllConnectivityFileTypes(std::vector<SpecFileDataFile*>& connectivityDataFilesOut)
+//{
+//    connectivityDataFilesOut.clear();
+//    
+//    for (std::vector<SpecFileDataFileTypeGroup*>::const_iterator iter = dataFileTypeGroups.begin();
+//         iter != dataFileTypeGroups.end();
+//         iter++) {
+//        SpecFileDataFileTypeGroup* dataFileTypeGroup = *iter;
+//        if (DataFileTypeEnum::isConnectivityDataType(dataFileTypeGroup->getDataFileType())) {
+//            const int32_t numFiles = dataFileTypeGroup->getNumberOfFiles();
+//            for (int32_t i = 0; i < numFiles; i++) {
+//                connectivityDataFilesOut.push_back(dataFileTypeGroup->getFileInformation(i));
+//            }
+//        }
+//    }
+//}
 
 /**
  * Set the selection status of a data file.
@@ -789,6 +778,32 @@ SpecFile::hasFilesWithRemotePathSelectedForLoading() const
     return false;
 }
 
+/**
+ * @return A vector containing all file names selected for loading.
+ */
+std::vector<AString>
+SpecFile::getAllDataFileNamesSelectedForLoading() const
+{
+    std::vector<AString> allFileNames;
+    
+    for (std::vector<SpecFileDataFileTypeGroup*>::const_iterator iter = dataFileTypeGroups.begin();
+         iter != dataFileTypeGroups.end();
+         iter++) {
+        SpecFileDataFileTypeGroup* dataFileTypeGroup = *iter;
+        
+        const int32_t numFiles = dataFileTypeGroup->getNumberOfFiles();
+        for (int32_t i = 0; i < numFiles; i++) {
+            if (dataFileTypeGroup->getFileInformation(i)->isLoadingSelected()) {
+                const AString filename = dataFileTypeGroup->getFileInformation(i)->getFileName();
+                allFileNames.push_back(filename);
+            }
+        }
+    }
+    
+    return allFileNames;
+}
+
+
 /**
  * @return A vector containing all file names.
  */
@@ -1370,7 +1385,9 @@ SpecFile::saveToScene(const SceneAttributes* sceneAttributes,
     std::set<const CaretDataFile*> displayedDataFiles;
     if ( ! allLoadedFilesFlag) {
         const std::vector<int32_t> tabIndicesForScene = sceneAttributes->getIndicesOfTabsForSavingToScene();
-        EventGetDisplayedDataFiles displayedFilesEvent(tabIndicesForScene);
+        const std::vector<int32_t> windowIndicesForScene = sceneAttributes->getIndicesOfWindowsForSavingToScene();
+        EventGetDisplayedDataFiles displayedFilesEvent(windowIndicesForScene,
+                                                       tabIndicesForScene);
         EventManager::get()->sendEvent(displayedFilesEvent.getPointer());
         displayedDataFiles = displayedFilesEvent.getDisplayedDataFiles();        
     }
@@ -1678,3 +1695,23 @@ SpecFile::addToDataFileContentInformation(DataFileContentInformation& dataFileIn
                                 + table.getInString());
 }
 
+/**
+ * Is the given file type allowed in the spec file?
+ *
+ * @param dataFileType
+ *     Type of data file.
+ * @return
+ *     True if allowed in spec file, else false.
+ */
+bool
+SpecFile::isDataFileTypeAllowedInSpecFile(const DataFileTypeEnum::Enum dataFileType)
+{
+    if (dataFileType == DataFileTypeEnum::CONNECTIVITY_DENSE_DYNAMIC) {
+        return false;
+    }
+    
+    return true;
+}
+
+
+
