@@ -96,6 +96,8 @@ namespace caret {
         int32_t getNumberOfFilesSelectedForSaving() const;
         
         std::vector<AString> getAllDataFileNames() const;
+
+        std::vector<AString> getAllDataFileNamesSelectedForLoading() const;
         
         bool areAllFilesSelectedForLoadingSceneFiles() const;
         
@@ -117,7 +119,7 @@ namespace caret {
         
         SpecFileDataFileTypeGroup* getDataFileTypeGroupByType(const DataFileTypeEnum::Enum dataFileType) const;
         
-        void getAllConnectivityFileTypes(std::vector<SpecFileDataFile*>& connectivityDataFiles);
+//        void getAllConnectivityFileTypes(std::vector<SpecFileDataFile*>& connectivityDataFiles);
         
         void setAllFilesSelectedForLoading(bool selectionStatus);
         
@@ -145,6 +147,8 @@ namespace caret {
         
         virtual void clearModified();
         
+        static bool isDataFileTypeAllowedInSpecFile(const DataFileTypeEnum::Enum dataFileType);
+        
         /** XML Tag for SpecFile element */
         static const AString XML_TAG_SPEC_FILE;
         
