@@ -37,6 +37,7 @@
 #include "OverlayViewController.h"
 #undef __OVERLAY_VIEW_CONTROLLER_DECLARE__
 
+#include "AnnotationColorBar.h"
 #include "CaretMappableDataFile.h"
 #include "EventDataFileReload.h"
 #include "EventGraphicsUpdateAllWindows.h"
@@ -373,6 +374,11 @@ OverlayViewController::fileComboBoxSelected(int indx)
 void
 OverlayViewController::mapIndexSpinBoxValueChanged(int indx)
 {
+    if (overlay == NULL)
+    {
+        //TSC: not sure how to put the displayed integer back to 0 where it starts when opening without data files
+        return;
+    }
     /*
      * Get the file that is selected from the file combo box
      */
@@ -499,8 +505,8 @@ OverlayViewController::colorBarActionTriggered(bool status)
     if (overlay == NULL) {
         return;
     }
-    
-    this->overlay->setPaletteDisplayEnabled(status);
+
+    this->overlay->getColorBar()->setDisplayed(status);
     
     this->updateGraphicsWindow();
 }
@@ -544,8 +550,9 @@ OverlayViewController::yokingGroupActivated()
    
     /*
      * Has yoking group changed?
+     * TSC: overlay can be null when opened without loaded files
      */
-    if (yokingGroup != overlay->getMapYokingGroup()) {
+    if (overlay != NULL && yokingGroup != overlay->getMapYokingGroup()) {
         validateYokingSelection();
     }
 }
@@ -664,7 +671,7 @@ OverlayViewController::updateViewController(Overlay* overlay)
     m_mapYokingGroupComboBox->setMapYokingGroup(overlay->getMapYokingGroup());
     
     this->colorBarAction->blockSignals(true);
-    this->colorBarAction->setChecked(overlay->isPaletteDisplayEnabled());
+    this->colorBarAction->setChecked(overlay->getColorBar()->isDisplayed());
     this->colorBarAction->blockSignals(false);
     
     this->opacityDoubleSpinBox->blockSignals(true);
@@ -709,9 +716,11 @@ OverlayViewController::updateViewController(Overlay* overlay)
     if (selectedFile != NULL) {
         FileInformation fileInfo(selectedFile->getFileName());
         fileComboBoxToolTip.append(":\n"
-                                + fileInfo.getFileName()
-                                + "\n"
-                                + fileInfo.getPathName());
+                                   + fileInfo.getFileName()
+                                   + "\n"
+                                   + fileInfo.getPathName()
+                                   + "\n\n"
+                                   + "Copy File Name/Path to Clipboard with Construction Menu");
         
         nameComboBoxToolTip.append(":\n"
                                    + this->mapNameComboBox->currentText());
