@@ -23,7 +23,6 @@
 
 
 #include "CaretObject.h"
-#include "XmlException.h"
 #include <stdint.h>
 
 #include <AString.h>
@@ -69,6 +68,8 @@ public:
             const double ty,
             const double tz);
 
+    void translate(const double txyz[3]);
+    
     void scale(
             const double sx,
             const double sy,
@@ -132,6 +133,8 @@ public:
 
     void multiplyPoint3(float p[3]) const;
     
+    void multiplyPoint3(double p[3]) const;
+    
     void multiplyPoint3X3(float p[3]) const;
 
     AString getDataSpaceName() const;
